"""ghettorecorder console module
"""
from ghettorecorder import ghetto_recorder


def main():
    # command line version; write:python ghetto_recorder.py
    ghetto_recorder.terminal_main()


if __name__ == '__main__':
    main()
