@import "https://fonts.googleapis.com/css?family=Saira+Extra+Condensed:700&amp;subset=latin-ext";

body, html {
	display:block;
    font-family: "Lato","proxima-nova","Helvetica Neue",Arial,sans-serif;
	font-weight: normal;
	font-size:16px;
	overflow-x: hidden;
	-webkit-text-size-adjust: 100%;

}

h1, h2, h3, h4, h5, h6 {
    margin-top: 0;
    font-weight: 700;
    font-family: "Lato","proxima-nova","Helvetica Neue",Arial,sans-serif;
}
